/*
   gssthread.c
   Copyright 2017 Greg Sydney-Smith
   
   2017-11-03 gss. Minimalist implimentation of unix pthread for z80sim.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

   THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
   INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
   FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
   DEVELOPERS AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
   OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
   OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
   ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <windows.h>
#include <stdio.h>
#include <conio.h>
#include "pthread.h"

static HANDLE td[8];
static DWORD  tdId[8];
static int    tdN = 0;

int pthread_create (pthread_t * tid,
                            const pthread_attr_t * attr,
                            void *(*start) (void *),
                            void *arg) {
    HANDLE hThread;
    DWORD threadID;

    if (tdN >= 8) return 0;
    hThread = CreateThread(NULL, // security attributes ( default if NULL )
                            0, // stack SIZE default if 0
                            (LPTHREAD_START_ROUTINE) start, // Start Address
                            NULL, // input data
                            0, // creational flag ( start if  0 )
                            &threadID); // thread ID
    td[tdN]= hThread;
    tdId[tdN]= threadID;
    return tdN++;
}

void pthread_exit (void *value_ptr) {
    // should do something but not used for now
}

// lets add a few Win specific bits ...
int os_sleep(int mS) {
    Sleep(mS);
    return 0;
}

static DWORD fdwSaveOldMode;

void int_on(void)
{
    HANDLE hStdin; 

    if ((hStdin = GetStdHandle(STD_INPUT_HANDLE))==INVALID_HANDLE_VALUE)
        printf("int_on: Invalid handle\n"); 
    if (! GetConsoleMode(hStdin, &fdwSaveOldMode) ) 
        printf("int_on: GetConsoleMode failed\n"); 
    if (!SetConsoleMode(hStdin,0)) // ENABLE_PROCESSED_INPUT)) // ^C goes to the prog
        printf("int_on: SetConsoleMode failed\n");
//    printf("int_on: done\n"); 
}

void int_off(void)
{
    HANDLE hStdin; 

    if ((hStdin = GetStdHandle(STD_INPUT_HANDLE))==INVALID_HANDLE_VALUE)
        printf("int_off: Invalid handle\n"); 
    if (!SetConsoleMode(hStdin,fdwSaveOldMode)) // ENABLE_PROCESSED_INPUT)) // ^C goes to the prog
        printf("int_off: SetConsoleMode failed\n");
//    printf("int_off: done\n"); 
}



